#!/usr/bin/perl

## Date and Version
my $version= '1.5';
my $version_date = '18_july_2012';

##################################################
## Documentation POD
##################################################

##################################################
## Modules
##################################################
## Perl modules
use strict;
use warnings;
use diagnostics;

use Getopt::Long;

# BioPerl modules
use Bio::SeqIO;

##################################################
## Globals variables
##################################################

# Initializations
my $DEBUG = 0;
my $Start_time_hr = humanReadableDate();
my %Parameters;
my $Files_to_merge = undef;

# Default values
my $Default_Rename_singlets = 'no';
my $Default_Remove_description = 'yes';
my $Default_Output_filename = 'ConcatContigs_' . time() . '.concat';

##################################################
## Manage options
##################################################

# Get options from command line
GetOptions ('rename_singlets|rsin:s'	=> \$Parameters{'Rename_singlets'},
			'remove_desc|rdesc:s'		=> \$Parameters{'Remove_description'},
			'output|out:s'				=> \$Parameters{'Output_filename'},
			'files=s@'					=> \$Files_to_merge
			);

# Check parameters
checkParameters(\%Parameters);

##################################################
## Main code
##################################################

# Welcome message
print "#############################################\n";
print "# Welcome in ConcatContigs.pl (Version $version) #\n"; 
print "#############################################\n";
print "Start time: " . $Start_time_hr . "\n\n";

print 'Rename singlets ? ' . $Parameters{'Rename_singlets'} . "\n";
print 'Remove sequence\'s description ? ' . $Parameters{'Remove_description'} . "\n\n";

print 'The following files will be merged:' . "\n\n";
foreach my $files (@{$Files_to_merge}) {
	print '  - ' . $files . "\n";
}

# Initializations
my $Counter=1;

# Creation of an output stream
my $SeqIO_out_object = Bio::SeqIO->new( -format => 'fasta', -file => '>' . $Parameters{'Output_filename'} );

print "\n" . 'Merging procedure in progress..' . "\n\n";

# Concatenate all input files into a single output file
foreach my $Input_file (@{$Files_to_merge}) {
	
	my $SeqIO_in_object = Bio::SeqIO->new( -file=> $Input_file , -format => 'fasta' );
	
	while (my $Current_Sequence = $SeqIO_in_object->next_seq()) {
		
		# Renames current sequence/contig if needed
		print STDERR $Current_Sequence->display_id();
		if ($Current_Sequence->display_id() =~ /[Contig|contig]/ || $Parameters{'Rename_singlets'} eq 'yes') {
			$Current_Sequence->display_id('Contig' . $Counter);
			$Counter++;
		}

		# Remove description if requested
		if ($Parameters{'Remove_description'} eq 'yes') {
			
			$Current_Sequence->desc('');
		}

		$SeqIO_out_object->write_seq($Current_Sequence);
	
	}
}

if (-e $Parameters{'Output_filename'} && !-z $Parameters{'Output_filename'}) {
	print 'All input files have been merged (' . $Parameters{'Output_filename'} . ')' . "\n";
} else {
	print ' --> No output file ! Something might have gone wrong...' . "\n\n";
}

print "\nStop time: " . humanReadableDate() . "\n";
print "#############################################\n";
print "#             End of execution              #\n"; 
print "#############################################\n";

##################################################
## Help/Check related functions
##################################################

sub checkParameters {
	
	# Recovers parameters
	my $Ref_to_parameters = shift;

	if (!defined($Ref_to_parameters->{'Rename_singlets'}))  {
		print 'Warning: The rename_singlets/rsin parameter has been omited ! Default value ("' . $Default_Rename_singlets . '") will be used...' . "\n\n";
		$Ref_to_parameters->{'Rename_singlets'} = $Default_Rename_singlets;
		
	} elsif ($Ref_to_parameters->{'Rename_singlets'} !~ /^(yes|no)$/i) {
		print 'Warning: The value (' . $Ref_to_parameters->{'Rename_singlets'} . ') of the rename_singlets/rsin parameter given to this module is not valid ! Default value ("' . $Default_Rename_singlets . '") will be used !' . "\n\n";
		$Ref_to_parameters->{'Rename_singlets'} = $Default_Rename_singlets;
	}
	
	if (!defined($Ref_to_parameters->{'Remove_description'}))  {
		print 'Warning: The remove_desc/rdesc parameter has been omited ! Default value ("' . $Default_Remove_description . '") will be used...' . "\n\n";
		$Ref_to_parameters->{'Remove_description'} = $Default_Remove_description;
		
	} elsif ($Ref_to_parameters->{'Remove_description'} !~ /^(yes|no)$/i) {
		print 'Warning: The value (' . $Ref_to_parameters->{'Remove_description'} . ') of the remove_desc/rdesc parameter given to this module is not valid ! Default value ("' . $Default_Remove_description . '") will be used !' . "\n\n";
		$Ref_to_parameters->{'Remove_description'} = $Default_Remove_description;
	}
	
	if (!defined($Ref_to_parameters->{'Output_filename'}))  {
		print 'Warning: The name of the main output file has been omited ! A default name (' . $Default_Output_filename . ') will be used !' . "\n\n";
		$Ref_to_parameters->{'Output_filename'} = $Default_Output_filename;
	}
	
	return 0;
}

##################################################
## Basic common functions
##################################################

sub humanReadableDate {
	
	# Recovers parameters
	my $time = shift || time;
	
	# Split time string
	my ($seconde, $minute, $heure, $jour, $mois, $annee, $jour_semaine, $jour_annee, $heure_hiver_ou_ete) = localtime($time);
	$mois  += 1;
	$annee += 1900;

	# On rajoute 0 si le chiffre est compris entre 1 et 9
	foreach ( $seconde, $minute, $heure, $jour, $mois, $annee ) { s/^(\d)$/0$1/; }
	
	return "$jour-$mois-$annee" . '_' . "$heure:$minute:$seconde";
}
